SET @OLD_UNIQUE_CHECKS=@@UNIQUE_CHECKS, UNIQUE_CHECKS=0;
SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0;
SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='TRADITIONAL,ALLOW_INVALID_DATES';


-- -----------------------------------------------------
-- Table `tms_user`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS `tms_user` (
  `UserNo` INT NOT NULL AUTO_INCREMENT ,
  `UserRef` VARCHAR(60) NULL ,
  `Name` VARCHAR(60) NULL ,
  `Company` VARCHAR(60) NULL ,
  `Email` VARCHAR(45) NULL ,
  `Info` LONGTEXT NULL ,
  PRIMARY KEY (`UserNo`) ,
  INDEX `ix_ref` (`UserRef` ASC) ,
  INDEX `ix_name` (`Name` ASC) ,
  INDEX `ix_email` (`Email` ASC) )
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8
COLLATE = utf8_general_ci;


-- -----------------------------------------------------
-- Table `tms_settings`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS `tms_settings` (
  `SettingsNo` INT NOT NULL AUTO_INCREMENT ,
  `EncryptionKeys` TEXT NULL ,
  `LicPassword` CHAR(64) NULL ,
  `DispFeedback` TINYINT(1) NULL ,
  `DispInfo1` TINYINT(1) NULL ,
  `DispInfo2` TINYINT(1) NULL ,
  `DispInfo3` TINYINT(1) NULL ,
  `InfoTitle1` VARCHAR(45) NULL ,
  `InfoTitle2` VARCHAR(45) NULL ,
  `InfoTitle3` VARCHAR(45) NULL ,
  PRIMARY KEY (`SettingsNo`) )
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8
COLLATE = utf8_general_ci;


-- -----------------------------------------------------
-- Table `tms_product`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS `tms_product` (
  `ProdNo` INT NOT NULL AUTO_INCREMENT ,
  `ProdID` VARCHAR(30) NULL ,
  `Desc` VARCHAR(80) NULL ,
  `CurrentVer` VARCHAR(20) NULL ,
  `TrialDays` INT NULL DEFAULT 10 ,
  `TrialOptions` VARCHAR(255) NULL ,
  `LicOptions` VARCHAR(255) NULL ,
  `OfflineTimeout` INT NULL DEFAULT 120 ,
  `ActivityTimeout` INT NULL DEFAULT 20 ,
  `Email` VARCHAR(45) NULL ,
  `EmailInstalls` TINYINT(1) NULL DEFAULT 0 ,
  `EmailUninstalls` TINYINT(1) NULL DEFAULT 0 ,
  `EmailFeedback` TINYINT(1) NULL DEFAULT 0 ,
  PRIMARY KEY (`ProdNo`) ,
  INDEX `ix_prodid` (`ProdID` ASC) )
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8
COLLATE = utf8_general_ci;


-- -----------------------------------------------------
-- Table `tms_license`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS `tms_license` (
  `LicNo` INT NOT NULL AUTO_INCREMENT ,
  `ProdNo` INT NULL ,
  `UserNo` INT NULL ,
  `IssueDate` DATETIME NULL ,
  `OrderRef` VARCHAR(45) NULL ,
  `Lic` VARCHAR(80) NULL ,
  `LicOptions` VARCHAR(255) NULL ,
  `DevID` CHAR(64) NULL ,
  `Status` CHAR(10) NULL ,
  `Type` CHAR(10) NULL ,
  `ExpDate` DATE NULL ,
  `MaxUsers` INT NULL ,
  `Notes` LONGTEXT NULL ,
  PRIMARY KEY (`LicNo`) ,
  INDEX `ix_lic` (`Lic` ASC) )
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8
COLLATE = utf8_general_ci;


-- -----------------------------------------------------
-- Table `tms_track`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS `tms_track` (
  `TrackNo` INT NOT NULL AUTO_INCREMENT ,
  `DevID` CHAR(64) NULL ,
  `ProdNo` INT NULL ,
  `Ver` VARCHAR(20) NULL ,
  `LicNo` INT NULL ,
  `InstallDate` DATETIME NULL ,
  `InstallLen` CHAR(16) NULL ,
  `FeedbackDate` DATETIME NULL ,
  `Feedback` LONGTEXT NULL ,
  `Name` VARCHAR(60) NULL ,
  `Email` VARCHAR(45) NULL ,
  `TrialStatus` CHAR(10) NULL ,
  `TrialExpDate` DATETIME NULL ,
  `ActiveTime` DATETIME NULL ,
  `Location` VARCHAR(255) NULL ,
  `Info1` VARCHAR(255) NULL ,
  `Info2` VARCHAR(255) NULL ,
  `Info3` VARCHAR(255) NULL ,
  PRIMARY KEY (`TrackNo`) ,
  INDEX `ix_lic` (`LicNo` ASC, `ActiveTime` DESC) )
ENGINE = InnoDB
DEFAULT CHARACTER SET = utf8
COLLATE = utf8_general_ci;



SET SQL_MODE=@OLD_SQL_MODE;
SET FOREIGN_KEY_CHECKS=@OLD_FOREIGN_KEY_CHECKS;
SET UNIQUE_CHECKS=@OLD_UNIQUE_CHECKS;

-- -----------------------------------------------------
-- Data for table `tms_settings`
-- -----------------------------------------------------
START TRANSACTION;
INSERT INTO `tms_settings` (`SettingsNo`, `EncryptionKeys`, `LicPassword`, `DispFeedback`, `DispInfo1`, `DispInfo2`, `DispInfo3`, `InfoTitle1`, `InfoTitle2`, `InfoTitle3`) VALUES (NULL, NULL, NULL, 0, 0, 0, 0, 'Info1', 'Info2', 'Info3');

COMMIT;

-- -----------------------------------------------------
-- Data for table `tms_product`
-- -----------------------------------------------------
START TRANSACTION;
INSERT INTO `tms_product` (`ProdNo`, `ProdID`, `Desc`, `CurrentVer`, `TrialDays`, `TrialOptions`, `LicOptions`, `OfflineTimeout`, `ActivityTimeout`, `Email`, `EmailInstalls`, `EmailUninstalls`, `EmailFeedback`) VALUES (NULL, 'Test1', 'Test Product 1', '1.0', 10, 'Default trial options', 'Default license options', 72, 60, NULL, 0, 0, 0);

COMMIT;
